
=head1 NAME

ll4.pm 

=head1 SYNOPSIS

Statistical library package to calculate the Loglikelihood ratio for 
trigrams. This package should be used with statistic.pl and rank.pl.

=head1 DESCRIPTION

The log-likelihood ratio measures the devitation between the observed data 
and what would be expected if <word1>, <word2>, <word3> and <word4>  were  
independent. The higher the score, the less evidence there is in favor of  
concluding that the words are independent. 

=head1 AUTHORS

Ted Pedersen <tpederse@d.umn.edu>

Bridget Thomson McInnes <bthomson@d.umn.edu>

=head1 BUGS

This measure currently only defined for 4gram data stored in 4x4x4x4
contingency table. 

=head1 SEE ALSO

Mailing List: http://groups.yahoo.com/ngram

=head1 COPYRIGHT

Copyright 2000-2004 by Ted Pedersen and Satanjeev Banerjee and Amruta 
Purandare

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=cut

## this module written by Amruta Purandare, based on ll.pm by Satanjeev 
## Banerjee

package ll4;

#  Make sure that measure4d.pm is available in the PATH. First
#  we check in the directory you are running from, and then we
#  look through the system path. If the module is not found anywhere
#  then abort. 

my $module = "measure4d.pm"; my $modulename = "measure4d.pm";

if( !( -f $modulename ) ) {
    my $found = 0;
    #  Check each of the PATHS to see if the module is there
    foreach (split(/:/, $ENV{PATH})) {
 	$module = $_ . "/" . $modulename;
	if ( -f $module ) { $found = 1; last; }
    }
    # if still not found anywhere, quit!
    if ( ! $found ) { print "Could not find $modulename.\n"; exit; }
}

# Include the module into the current package.    

require $module;
require Exporter;

@ISA = qw ( Exporter );
@EXPORT = qw (initializeStatistic getStatisticName calculateStatistic errorCode errorString);

# function to set up various variables before the actual computation
# starts. also to check if we are being given bigrams, and if our
# frequency combinations are enough to do the computation
sub initializeStatistic
{    
    measure4d::initializeStatistic(@_);
}

# function to calculate the ll value!
sub calculateStatistic
{
    #  The parameters passed into calculateStatistic
    #  need to be passed into getObservedValues for it 
    #  to calculate the Observed values. If there is a 
    #  problem with the observed values, this function 
    #  will return, causing the measure to abort its 
    #  computations and return with a zero code.
    
    if( !( ($n1111, $n1112, $n1121, $n1122, $n1211, $n1212, $n1221, $n1222, 
	    $n2111, $n2112, $n2121, $n2122, $n2211, $n2212, $n2221, $n2222) 
	   = measure4d::getObservedValues(@_) ) ) {
	return(0);
    }
    
    #  Check the expected values to ensure that they are not 
    #  equal to zero. Also, the quotient of the observed and 
    #  expected value should not be less than zero. If there 
    #  is a problem with the expected values, this function
    #  will return, causing the measure to abort and return
    #  with a zero code.
    
    if( !( ($m1111, $m1112, $m1121, $m1122, $m1211, $m1212, $m1221, $m1222, 
	    $m2111, $m2112, $m2121, $m2122, $m2211, $m2212, $m2221, $m2222) 
	   = measure4d::getExpectedValues() ) ) {
	return(0);
    }
    
    #  Get the total number of bigrams; no need to check 
    #  if this is correct because this value was already 
    #  checked and computed in measuredd::initalizeStatistic.
    #  getTotalTrigrams simply reads an existing value. 
    
    my $npppp = measure4d::getTotal4grams();
    
    #  Now for the calculations!!
   
    $logLikelihood = 0;
 
    # dont want ($n111 / $e111) to be 0 or less! flag error if so!
    if ( $n1111 ) { { $logLikelihood += $n1111 * log ( $n1111 / $m1111 ); }}
    if ( $n1112 ) { { $logLikelihood += $n1112 * log ( $n1112 / $m1112 ); }}
    if ( $n1121 ) { { $logLikelihood += $n1121 * log ( $n1121 / $m1121 ); }}
    if ( $n1122 ) { { $logLikelihood += $n1122 * log ( $n1122 / $m1122 ); }}
    if ( $n1211 ) { { $logLikelihood += $n1211 * log ( $n1211 / $m1211 ); }}
    if ( $n1212 ) { { $logLikelihood += $n1212 * log ( $n1212 / $m1212 ); }}
    if ( $n1221 ) { { $logLikelihood += $n1221 * log ( $n1221 / $m1221 ); }}
    if ( $n1222 ) { { $logLikelihood += $n1222 * log ( $n1222 / $m1222 ); }}
    if ( $n2111 ) { { $logLikelihood += $n2111 * log ( $n2111 / $m2111 ); }}
    if ( $n2112 ) { { $logLikelihood += $n2112 * log ( $n2112 / $m2112 ); }}
    if ( $n2121 ) { { $logLikelihood += $n2121 * log ( $n2121 / $m2121 ); }}
    if ( $n2122 ) { { $logLikelihood += $n2122 * log ( $n2122 / $m2122 ); }}
    if ( $n2211 ) { { $logLikelihood += $n2211 * log ( $n2211 / $m2211 ); }}
    if ( $n2212 ) { { $logLikelihood += $n2212 * log ( $n2212 / $m2212 ); }}
    if ( $n2221 ) { { $logLikelihood += $n2221 * log ( $n2221 / $m2221 ); }}
    if ( $n2222 ) { { $logLikelihood += $n2222 * log ( $n2222 / $m2222 ); }}
  

    return (2 * $logLikelihood );
}


# function to return the error code of the last operation and reset
# error code. useful if the error can be recovered from!
sub errorCode 
{ 
    return measure4d::errorCode();
}

# function to return the error message of the last operation and reset
# the message string. useful if error can be recovered from!
sub errorString
{
    return measure4d::errorString();
}

# function to return the name of this statistic
sub getStatisticName
{
    return "Loglikelihood4";
}

1;

